from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithRequestPatchRequestBody(Parsable):
    # The callback URL specified for the data request. If specified, the Data Connection service calls the URL each time a job executes for the request. The service sends a POST request that provides job execution information. The JSON payload in the POST request contains the following: ``{ ``accountId``: ``account_id``, ``requestId``: ``request_id``, ``jobId``: ``data_connector_job_id``, ``state``: ``complete``, ``success``: true or false }``.If not specified, the Data Connection service does not provide a callback.
    callback_url: Optional[str] = None
    # The user-entered description of this data request.
    description: Optional[str] = None
    # The date and time when a one-time job execution or a recurring interval schedule begins, presented in ISO 8601 format.
    effective_from: Optional[datetime.datetime] = None
    # The date and time when the recurring interval schedule ends, presented in ISO 8601 format.
    effective_to: Optional[datetime.datetime] = None
    # The end date and time for the data extraction, in ISO 8601 format.This field applies only to schemas supporting date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.Additional notes on using ``startDate`` and ``endDate``:- If you provide only ``startDate`` or ``endDate`` (but not both), Data Connector uses that single date for both ``startDate`` and ``endDate``.- If you request more than the Maximum Date Range Allowed for an extraction, the default date range as documented in the schema documentation is returned.- For the ``activities`` service group, data replication can be delayed up to 20 minutes, so your requests should account for that delay.
    end_date: Optional[str] = None
    # The data request's active/inactive status. Possible values: ``true`` the request is active; ``false`` the request is inactive.
    is_active: Optional[bool] = None
    # (Legacy): A single project ID for the data request. Superseded by ``projectIdList``.
    project_id: Optional[str] = None
    # A list of up to 50 project IDs for the data request, which can include a single project or multiple projects. If ``projectId`` is also included, ``projectIdList`` takes precedence. Required for users with project admin permissions. Optional for users with executive overview permissions, who by default receive data for all projects unless ``projectIdList`` is provided.
    project_id_list: Optional[str] = None
    # The number of ``scheduleInterval`` units to wait between job execution for the request. For example, a ``scheduleInterval`` value of ``WEEK`` and a ``reoccuringInterval`` value of ``2`` means the job will run every two weeks.
    reoccuring_interval: Optional[int] = None
    # The scheduling interval unit for jobs spawned by this data request. This value is multiplied by the ``reoccurringInterval`` attribute to specify the length of the recurring interval at which jobs run. Possible values:- ``ONE_TIME``: Run the job only once- ``DAY``: Set the recurring job interval in days- ``WEEK``: Set the recurring job interval in weeks- ``MONTH``: Set the recurring job interval in months- ``YEAR``: Set the recurring job interval in yearsNote that recurring jobs start at the day and time when the request first spawns a job. This may be at the date and time specified in the attribute ``effectiveFrom``.This value is required.
    schedule_interval: Optional[str] = None
    # Send a notification email to the user upon job completion. Values: true or false (default is true)
    send_email: Optional[bool] = None
    # The service groups from which to extract data, separated by commas.Possible values: ``all``, ``activities``, ``admin``, ``assets``, ``checklists``, ``cost``, ``dailylogs``, ``forms``, ``iq``, ``issues``, ``locations``, ``markups``, ``meetingminutes``, ``photos``, ``relationships``, ``reviews``, ``rfis``, ``schedule``, ``sheets``, ``submittals``, ``submittalsacc``, ``transmittals``.Note that the ``admin`` service includes both project and account admin, and ``all`` produces an extract containing all currently available service groups.
    service_groups: Optional[list[str]] = None
    # The start date and time for the data extraction, in ISO 8601 format.This field applies only to schemas supporting date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.Additional notes on using ``startDate`` and ``endDate``:- If you provide only ``startDate`` or ``endDate`` (but not both), Data Connector uses that single date for both ``startDate`` and ``endDate``.- If you request more than the Maximum Date Range Allowed for an extraction, the default date range as documented in the schema documentation is returned.- For the ``activities`` service group, data replication can be delayed up to 20 minutes, so your requests should account for that delay.
    start_date: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRequestPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRequestPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRequestPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "callbackUrl": lambda n : setattr(self, 'callback_url', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "effectiveFrom": lambda n : setattr(self, 'effective_from', n.get_datetime_value()),
            "effectiveTo": lambda n : setattr(self, 'effective_to', n.get_datetime_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "projectIdList": lambda n : setattr(self, 'project_id_list', n.get_str_value()),
            "reoccuringInterval": lambda n : setattr(self, 'reoccuring_interval', n.get_int_value()),
            "scheduleInterval": lambda n : setattr(self, 'schedule_interval', n.get_str_value()),
            "sendEmail": lambda n : setattr(self, 'send_email', n.get_bool_value()),
            "serviceGroups": lambda n : setattr(self, 'service_groups', n.get_collection_of_primitive_values(str)),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("callbackUrl", self.callback_url)
        writer.write_str_value("description", self.description)
        writer.write_datetime_value("effectiveFrom", self.effective_from)
        writer.write_datetime_value("effectiveTo", self.effective_to)
        writer.write_str_value("endDate", self.end_date)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_str_value("projectId", self.project_id)
        writer.write_str_value("projectIdList", self.project_id_list)
        writer.write_int_value("reoccuringInterval", self.reoccuring_interval)
        writer.write_str_value("scheduleInterval", self.schedule_interval)
        writer.write_bool_value("sendEmail", self.send_email)
        writer.write_collection_of_primitive_values("serviceGroups", self.service_groups)
        writer.write_str_value("startDate", self.start_date)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class SearchGetResponse(Parsable):
    # The account_id property
    account_id: Optional[UUID] = None
    # The address_line_1 property
    address_line_1: Optional[str] = None
    # The address_line_2 property
    address_line_2: Optional[str] = None
    # The city property
    city: Optional[str] = None
    # The country property
    country: Optional[str] = None
    # The description property
    description: Optional[str] = None
    # The erp_id property
    erp_id: Optional[str] = None
    # The id property
    id: Optional[UUID] = None
    # The name property
    name: Optional[str] = None
    # The phone property
    phone: Optional[str] = None
    # The postal_code property
    postal_code: Optional[str] = None
    # The state_or_province property
    state_or_province: Optional[str] = None
    # The tax_id property
    tax_id: Optional[str] = None
    # The trade property
    trade: Optional[str] = None
    # The website_url property
    website_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SearchGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SearchGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SearchGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "account_id": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "address_line_1": lambda n : setattr(self, 'address_line_1', n.get_str_value()),
            "address_line_2": lambda n : setattr(self, 'address_line_2', n.get_str_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "erp_id": lambda n : setattr(self, 'erp_id', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "phone": lambda n : setattr(self, 'phone', n.get_str_value()),
            "postal_code": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "state_or_province": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "tax_id": lambda n : setattr(self, 'tax_id', n.get_str_value()),
            "trade": lambda n : setattr(self, 'trade', n.get_str_value()),
            "website_url": lambda n : setattr(self, 'website_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("account_id", self.account_id)
        writer.write_str_value("address_line_1", self.address_line_1)
        writer.write_str_value("address_line_2", self.address_line_2)
        writer.write_str_value("city", self.city)
        writer.write_str_value("country", self.country)
        writer.write_str_value("description", self.description)
        writer.write_str_value("erp_id", self.erp_id)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("phone", self.phone)
        writer.write_str_value("postal_code", self.postal_code)
        writer.write_str_value("state_or_province", self.state_or_province)
        writer.write_str_value("tax_id", self.tax_id)
        writer.write_str_value("trade", self.trade)
        writer.write_str_value("website_url", self.website_url)
    




from enum import Enum

class IssuesGetResponse_results_customAttributes_type(str, Enum):
    Numeric = "numeric",
    Paragraph = "paragraph",
    List_ = "list",
    Dropdown = "dropdown",
    Text = "text",




from enum import Enum

class WithTakeoffTypeGetResponse_tool(str, Enum):
    COUNT = "COUNT",
    DISTANCE = "DISTANCE",
    AREA = "AREA",
    SELECT = "SELECT",
    LINEAR = "LINEAR",
    BIM = "BIM",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .uploads_post_request_body_files_storage_type import UploadsPostRequestBody_files_storageType

@dataclass
class UploadsPostRequestBody_files(Parsable):
    # The name of the file.Currently the Sheets tool only supports PDF files.To create a storage URN, call `POST storage </en/docs/acc/v1/reference/http/sheets-storage-POST/>`_.Max length: 255
    name: Optional[str] = None
    # The storage type of the file. Will always be ``OSS``.
    storage_type: Optional[UploadsPostRequestBody_files_storageType] = None
    # The storage URN of the file.
    storage_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UploadsPostRequestBody_files:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UploadsPostRequestBody_files
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UploadsPostRequestBody_files()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .uploads_post_request_body_files_storage_type import UploadsPostRequestBody_files_storageType

        from .uploads_post_request_body_files_storage_type import UploadsPostRequestBody_files_storageType

        fields: dict[str, Callable[[Any], None]] = {
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "storageType": lambda n : setattr(self, 'storage_type', n.get_enum_value(UploadsPostRequestBody_files_storageType)),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_enum_value("storageType", self.storage_type)
        writer.write_str_value("storageUrn", self.storage_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .workflows_get_response_results_approval_status_options_value import WorkflowsGetResponse_results_approvalStatusOptions_value

@dataclass
class WorkflowsGetResponse_results_approvalStatusOptions(Parsable):
    # Indicates whether the approval status is a built-in option.``true``: the status is built in (e.g., ``APPROVED``, ``REJECTED``).``false``: the status is a custom option created by a user.
    built_in: Optional[bool] = None
    # The unique identifier of this approval status entry in the workflow.
    id: Optional[UUID] = None
    # The display name shown in the UI. It m ust be unique across all status options (built-in and custom).Maximum length: 255 characters.Max length: 255
    label: Optional[str] = None
    # The value representing the approval outcome. Possible values: ``APPROVED``, ``REJECTED``.
    value: Optional[WorkflowsGetResponse_results_approvalStatusOptions_value] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsGetResponse_results_approvalStatusOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsGetResponse_results_approvalStatusOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsGetResponse_results_approvalStatusOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflows_get_response_results_approval_status_options_value import WorkflowsGetResponse_results_approvalStatusOptions_value

        from .workflows_get_response_results_approval_status_options_value import WorkflowsGetResponse_results_approvalStatusOptions_value

        fields: dict[str, Callable[[Any], None]] = {
            "builtIn": lambda n : setattr(self, 'built_in', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_enum_value(WorkflowsGetResponse_results_approvalStatusOptions_value)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("builtIn", self.built_in)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("label", self.label)
        writer.write_enum_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .steps_get_response_results_permitted_actions_fields import StepsGetResponse_results_permittedActions_fields

@dataclass
class StepsGetResponse_results_permittedActions(Parsable):
    # A mapping of field names to lists of possible values for each field. If a field does not require a specific value, it will be returned as an empty array.
    fields: Optional[StepsGetResponse_results_permittedActions_fields] = None
    # The ID of the action in the format ``type_of_object::action``. For example, ``Step::partial_update``.
    id: Optional[str] = None
    # A list of fields required to perform specific actions on a step, such as ``tasks`` for the ``Step::partial_update`` action. The required fields depend on the user's role and the action.
    mandatory_fields: Optional[list[str]] = None
    # Not relevant
    transitions: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StepsGetResponse_results_permittedActions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StepsGetResponse_results_permittedActions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StepsGetResponse_results_permittedActions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .steps_get_response_results_permitted_actions_fields import StepsGetResponse_results_permittedActions_fields

        from .steps_get_response_results_permitted_actions_fields import StepsGetResponse_results_permittedActions_fields

        fields: dict[str, Callable[[Any], None]] = {
            "fields": lambda n : setattr(self, 'fields', n.get_object_value(StepsGetResponse_results_permittedActions_fields)),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "mandatoryFields": lambda n : setattr(self, 'mandatory_fields', n.get_collection_of_primitive_values(str)),
            "transitions": lambda n : setattr(self, 'transitions', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("fields", self.fields)
        writer.write_str_value("id", self.id)
        writer.write_collection_of_primitive_values("mandatoryFields", self.mandatory_fields)
        writer.write_collection_of_primitive_values("transitions", self.transitions)
    




from enum import Enum

class FormsPostResponse_customValues_valueName(str, Enum):
    TextVal = "textVal",
    ToggleVal = "toggleVal",
    ArrayVal = "arrayVal",
    NumberVal = "numberVal",
    ChoiceVal = "choiceVal",
    DateVal = "dateVal",
    SvgVal = "svgVal",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .reviews_get_response_results_next_action_by_candidates import ReviewsGetResponse_results_nextActionBy_candidates
    from .reviews_get_response_results_next_action_by_claimed_by import ReviewsGetResponse_results_nextActionBy_claimedBy

@dataclass
class ReviewsGetResponse_results_nextActionBy(Parsable):
    """
    Information about the claimers and candidates responsible for the current step.
    """
    # Information about the users, roles, and companies who are eligible to take the next action in this step.
    candidates: Optional[ReviewsGetResponse_results_nextActionBy_candidates] = None
    # A list of users who have already claimed the current step.
    claimed_by: Optional[list[ReviewsGetResponse_results_nextActionBy_claimedBy]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsGetResponse_results_nextActionBy:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsGetResponse_results_nextActionBy
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsGetResponse_results_nextActionBy()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .reviews_get_response_results_next_action_by_candidates import ReviewsGetResponse_results_nextActionBy_candidates
        from .reviews_get_response_results_next_action_by_claimed_by import ReviewsGetResponse_results_nextActionBy_claimedBy

        from .reviews_get_response_results_next_action_by_candidates import ReviewsGetResponse_results_nextActionBy_candidates
        from .reviews_get_response_results_next_action_by_claimed_by import ReviewsGetResponse_results_nextActionBy_claimedBy

        fields: dict[str, Callable[[Any], None]] = {
            "candidates": lambda n : setattr(self, 'candidates', n.get_object_value(ReviewsGetResponse_results_nextActionBy_candidates)),
            "claimedBy": lambda n : setattr(self, 'claimed_by', n.get_collection_of_object_values(ReviewsGetResponse_results_nextActionBy_claimedBy)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("candidates", self.candidates)
        writer.write_collection_of_object_values("claimedBy", self.claimed_by)
    




from enum import Enum

class ClosedPostResponse_reason(str, Enum):
    OTHER = "OTHER",
    VALID_INTERFACE = "VALID_INTERFACE",
    VALID_PENETRATION = "VALID_PENETRATION",
    MINIMAL_OVERLAP = "MINIMAL_OVERLAP",
    ITEM_CAN_FLEX = "ITEM_CAN_FLEX",
    MODEL_INACCURACY = "MODEL_INACCURACY",
    FIELD_FIX = "FIELD_FIX",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_item_patch_request_body_manager_type import WithItemPatchRequestBody_managerType
    from .with_item_patch_request_body_priority import WithItemPatchRequestBody_priority
    from .with_item_patch_request_body_subcontractor_type import WithItemPatchRequestBody_subcontractorType
    from .with_item_patch_request_body_watchers import WithItemPatchRequestBody_watchers

@dataclass
class WithItemPatchRequestBody(Parsable):
    # A custom number to assign to the submittal item. Before updating an item, check what the custom numbering type is for the current project by calling `GET metadata </en/docs/acc/v1/reference/http/submittals-metadata-GET/>`_ to get the custom identifier sequence type.Submittals support two custom numbering types:- Global Numbering: The custom identifier only contains the sequential number.- Spec Section Numbering: When the project uses spec section numbering for custom numbering, ensure that only the sequential number part of the custom identifier is sent in the request, excluding the spec ID.To find the next available custom number in the sequence, call `GET items:next-custom-identifier </en/docs/acc/v1/reference/http/submittals-itemsnext-custom-identifier-GET/>`_.To verify whether the custom number has not been used and is in a supported format, call `POST items:validate-custom-identifier </en/docs/acc/v1/reference/http/submittals-itemsvalidate-custom-identifier-POST/>`_.For more information on custom numbering, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Custom_Numbering>`_.
    custom_identifier: Optional[str] = None
    # A description of the submittal item to add context about its purpose.
    description: Optional[str] = None
    # The duration (in days) from the approval of the submittal to the delivery of materials or products to the construction site.
    lead_time: Optional[int] = None
    # The Autodesk ID (``autodeskId``) or Group ID (``memberGroupId``) specifying the manager to assign to the submittal item. The manager could be a user, role, or a company. Note that the user must be defined as a manager in the project. You can verify this by calling `GET mappings </en/docs/acc/v1/reference/http/submittals-mappings-GET/>`_.To get the possible values for a manager, call `GET mappings </en/docs/acc/v1/reference/http/submittals-mappings-GET/>`_ and check the autodeskId. This returns a list of all the users, roles, and companies that are managers in the project. Call `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_ to verify the actual name of the user. If a copmany is a manager, call `GET companies <en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_ to verify the actual name of the company. Note that we do not currently support verifying names of roles.
    manager: Optional[str] = None
    # The date by which the manager is expected to prepare the submittal item for review to the reviewer, in the following format: YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    manager_due_date: Optional[str] = None
    # The type of manager. This attribute is required when a manager is specified. To determine the type of manager, call `GET mappings </en/docs/acc/v1/reference/http/submittals-mappings-GET/>`_ and check the user type.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    manager_type: Optional[WithItemPatchRequestBody_managerType] = None
    # The ID of the package associated with the submittal item. This attribute is optional and is used for workflows involving attachments.To retrieve package details, call `GET packages </en/docs/acc/v1/reference/http/submittals-packages-GET/>`_.
    package_id: Optional[UUID] = None
    # The priority level of the submittal item.Possible values:- ``Low``- ``Normal``- ``High``When the priority is not set, the default value is ``Normal``.
    priority: Optional[WithItemPatchRequestBody_priority] = None
    # The date when the manager closed and distributed the submittal item, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    published_date: Optional[datetime.datetime] = None
    # The timestamp marking when the item transitioned from the ``rev`` state ``(Open - In Review)`` to the ``mgr-2`` state ``(close and distribute)``, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    received_from_review: Optional[datetime.datetime] = None
    # The date and time when the submittal was received back from the subcontractor after review, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    received_from_submitter: Optional[datetime.datetime] = None
    # The date by which approval for the submittal is required, in the following format: YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    required_approval_date: Optional[str] = None
    # The date the responsible contractor needs to submit the submittal to the submittal manager, in the following format: YYYY-MM-DD (ISO 8601) in UTC. For example, ``2018-02-15``.
    required_date: Optional[str] = None
    # The date when the materials are expected to arrive on the site, in the following format: YYYY-MM-DD (ISO 8601) in UTC. For example, ``2018-02-15``.
    required_on_job_date: Optional[str] = None
    # The date and time when the submittal item was transitioned to the ``rev`` state ``(Open - In review)``, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    sent_to_review: Optional[datetime.datetime] = None
    # The date and time when the submittal was sent to the subcontrator for review, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    sent_to_submitter: Optional[datetime.datetime] = None
    # The unique identifier (UUID) of the spec section to assign to the submittal item, which can be obtained by calling `GET specs </en/docs/acc/v1/reference/http/submittals-specs-GET/>`_.
    spec_id: Optional[UUID] = None
    # The Autodesk ID of the subcontractor to assign to the submittal item. When updating a submittal item as a manager in the UI, you will see ‘assign to responsible contractor’. The subcontractor can be a user (``autodeskId``), role (``memberGroupId``), or company (``memberGroupId``).For users, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.For companies, call `GET companies <en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_.Note that we do not currently support finding details about roles for a project.
    subcontractor: Optional[str] = None
    # The type of subcontractor. This attribute is required when a subcontrator is specified.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    subcontractor_type: Optional[WithItemPatchRequestBody_subcontractorType] = None
    # The date by which the subcontractor is expected to submit the submittal to the manager, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``.
    submitter_due_date: Optional[str] = None
    # Define the subsection details of the submittal item. For example, ``1.05-B``.
    subsection: Optional[str] = None
    # The title of the submittal item defined by the user when updating the submittal item.
    title: Optional[str] = None
    # The ID of the item type to assign to the submittal item. To retrieve the type ID, call `GET item-types </en/docs/acc/v1/reference/http/submittals-item-types-GET/>`_ or `GET item-types/:id </en/docs/acc/v1/reference/http/submittals-item-types-id-GET/>`_.
    type_id: Optional[UUID] = None
    # A list of project watchers, who can be individual users, roles, or companies.
    watchers: Optional[list[WithItemPatchRequestBody_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithItemPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithItemPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithItemPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_item_patch_request_body_manager_type import WithItemPatchRequestBody_managerType
        from .with_item_patch_request_body_priority import WithItemPatchRequestBody_priority
        from .with_item_patch_request_body_subcontractor_type import WithItemPatchRequestBody_subcontractorType
        from .with_item_patch_request_body_watchers import WithItemPatchRequestBody_watchers

        from .with_item_patch_request_body_manager_type import WithItemPatchRequestBody_managerType
        from .with_item_patch_request_body_priority import WithItemPatchRequestBody_priority
        from .with_item_patch_request_body_subcontractor_type import WithItemPatchRequestBody_subcontractorType
        from .with_item_patch_request_body_watchers import WithItemPatchRequestBody_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "customIdentifier": lambda n : setattr(self, 'custom_identifier', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "leadTime": lambda n : setattr(self, 'lead_time', n.get_int_value()),
            "manager": lambda n : setattr(self, 'manager', n.get_str_value()),
            "managerDueDate": lambda n : setattr(self, 'manager_due_date', n.get_str_value()),
            "managerType": lambda n : setattr(self, 'manager_type', n.get_enum_value(WithItemPatchRequestBody_managerType)),
            "packageId": lambda n : setattr(self, 'package_id', n.get_uuid_value()),
            "priority": lambda n : setattr(self, 'priority', n.get_enum_value(WithItemPatchRequestBody_priority)),
            "publishedDate": lambda n : setattr(self, 'published_date', n.get_datetime_value()),
            "receivedFromReview": lambda n : setattr(self, 'received_from_review', n.get_datetime_value()),
            "receivedFromSubmitter": lambda n : setattr(self, 'received_from_submitter', n.get_datetime_value()),
            "requiredApprovalDate": lambda n : setattr(self, 'required_approval_date', n.get_str_value()),
            "requiredDate": lambda n : setattr(self, 'required_date', n.get_str_value()),
            "requiredOnJobDate": lambda n : setattr(self, 'required_on_job_date', n.get_str_value()),
            "sentToReview": lambda n : setattr(self, 'sent_to_review', n.get_datetime_value()),
            "sentToSubmitter": lambda n : setattr(self, 'sent_to_submitter', n.get_datetime_value()),
            "specId": lambda n : setattr(self, 'spec_id', n.get_uuid_value()),
            "subcontractor": lambda n : setattr(self, 'subcontractor', n.get_str_value()),
            "subcontractorType": lambda n : setattr(self, 'subcontractor_type', n.get_enum_value(WithItemPatchRequestBody_subcontractorType)),
            "submitterDueDate": lambda n : setattr(self, 'submitter_due_date', n.get_str_value()),
            "subsection": lambda n : setattr(self, 'subsection', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "typeId": lambda n : setattr(self, 'type_id', n.get_uuid_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(WithItemPatchRequestBody_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("customIdentifier", self.custom_identifier)
        writer.write_str_value("description", self.description)
        writer.write_int_value("leadTime", self.lead_time)
        writer.write_str_value("manager", self.manager)
        writer.write_str_value("managerDueDate", self.manager_due_date)
        writer.write_enum_value("managerType", self.manager_type)
        writer.write_uuid_value("packageId", self.package_id)
        writer.write_enum_value("priority", self.priority)
        writer.write_datetime_value("publishedDate", self.published_date)
        writer.write_datetime_value("receivedFromReview", self.received_from_review)
        writer.write_datetime_value("receivedFromSubmitter", self.received_from_submitter)
        writer.write_str_value("requiredApprovalDate", self.required_approval_date)
        writer.write_str_value("requiredDate", self.required_date)
        writer.write_str_value("requiredOnJobDate", self.required_on_job_date)
        writer.write_datetime_value("sentToReview", self.sent_to_review)
        writer.write_datetime_value("sentToSubmitter", self.sent_to_submitter)
        writer.write_uuid_value("specId", self.spec_id)
        writer.write_str_value("subcontractor", self.subcontractor)
        writer.write_enum_value("subcontractorType", self.subcontractor_type)
        writer.write_str_value("submitterDueDate", self.submitter_due_date)
        writer.write_str_value("subsection", self.subsection)
        writer.write_str_value("title", self.title)
        writer.write_uuid_value("typeId", self.type_id)
        writer.write_collection_of_object_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ClosedPostResponse_clashData_clashInstances(Parsable):
    # The clash ID in the model set version.
    cid: Optional[int] = None
    # The left-hand-side document ID.
    ldid: Optional[int] = None
    # The left-hand-side object ID.
    loid: Optional[int] = None
    # The left-hand-side viewable ID.
    lvid: Optional[int] = None
    # The right-hand-side document ID.
    rdid: Optional[int] = None
    # The right-hand-side object ID.
    roid: Optional[int] = None
    # The left-hand-side viewable ID.
    rvid: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClosedPostResponse_clashData_clashInstances:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClosedPostResponse_clashData_clashInstances
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClosedPostResponse_clashData_clashInstances()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "cid": lambda n : setattr(self, 'cid', n.get_int_value()),
            "ldid": lambda n : setattr(self, 'ldid', n.get_int_value()),
            "loid": lambda n : setattr(self, 'loid', n.get_int_value()),
            "lvid": lambda n : setattr(self, 'lvid', n.get_int_value()),
            "rdid": lambda n : setattr(self, 'rdid', n.get_int_value()),
            "roid": lambda n : setattr(self, 'roid', n.get_int_value()),
            "rvid": lambda n : setattr(self, 'rvid', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("cid", self.cid)
        writer.write_int_value("ldid", self.ldid)
        writer.write_int_value("loid", self.loid)
        writer.write_int_value("lvid", self.lvid)
        writer.write_int_value("rdid", self.rdid)
        writer.write_int_value("roid", self.roid)
        writer.write_int_value("rvid", self.rvid)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .workflows_post_request_body_additional_approval_status_options import WorkflowsPostRequestBody_additionalApprovalStatusOptions
    from .workflows_post_request_body_additional_options import WorkflowsPostRequestBody_additionalOptions
    from .workflows_post_request_body_copy_files_options import WorkflowsPostRequestBody_copyFilesOptions
    from .workflows_post_request_body_steps import WorkflowsPostRequestBody_steps

@dataclass
class WorkflowsPostRequestBody(Parsable):
    # Custom approval statuses to add in addition to the built-in statuses (``APPROVED``, ``REJECTED``). You can define up to 50.
    additional_approval_status_options: Optional[list[WorkflowsPostRequestBody_additionalApprovalStatusOptions]] = None
    # Workflow-level settings that control whether the initiator can modify certain fields when starting a review.
    additional_options: Optional[WorkflowsPostRequestBody_additionalOptions] = None
    # (``Copy approved files`` in the UI) The configuration for copying approved files to a target folder when the review is complete.
    copy_files_options: Optional[WorkflowsPostRequestBody_copyFilesOptions] = None
    # A description of the workflow.Max length: 4096
    description: Optional[str] = None
    # The name of the workflow. It must be unique within the project.Max length: 255
    name: Optional[str] = None
    # A custom note associated with the workflow. Visible to all reviewers during the review process.Max length: 4096
    notes: Optional[str] = None
    # Defines the sequence of steps in the workflow. Steps run in the order they appear.
    steps: Optional[list[WorkflowsPostRequestBody_steps]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflows_post_request_body_additional_approval_status_options import WorkflowsPostRequestBody_additionalApprovalStatusOptions
        from .workflows_post_request_body_additional_options import WorkflowsPostRequestBody_additionalOptions
        from .workflows_post_request_body_copy_files_options import WorkflowsPostRequestBody_copyFilesOptions
        from .workflows_post_request_body_steps import WorkflowsPostRequestBody_steps

        from .workflows_post_request_body_additional_approval_status_options import WorkflowsPostRequestBody_additionalApprovalStatusOptions
        from .workflows_post_request_body_additional_options import WorkflowsPostRequestBody_additionalOptions
        from .workflows_post_request_body_copy_files_options import WorkflowsPostRequestBody_copyFilesOptions
        from .workflows_post_request_body_steps import WorkflowsPostRequestBody_steps

        fields: dict[str, Callable[[Any], None]] = {
            "additionalApprovalStatusOptions": lambda n : setattr(self, 'additional_approval_status_options', n.get_collection_of_object_values(WorkflowsPostRequestBody_additionalApprovalStatusOptions)),
            "additionalOptions": lambda n : setattr(self, 'additional_options', n.get_object_value(WorkflowsPostRequestBody_additionalOptions)),
            "copyFilesOptions": lambda n : setattr(self, 'copy_files_options', n.get_object_value(WorkflowsPostRequestBody_copyFilesOptions)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(WorkflowsPostRequestBody_steps)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("additionalApprovalStatusOptions", self.additional_approval_status_options)
        writer.write_object_value("additionalOptions", self.additional_options)
        writer.write_object_value("copyFilesOptions", self.copy_files_options)
        writer.write_str_value("description", self.description)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_collection_of_object_values("steps", self.steps)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .versions_batch_get_post_response_results_approval_status import VersionsBatchGetPostResponse_results_approvalStatus
    from .versions_batch_get_post_response_results_custom_attributes import VersionsBatchGetPostResponse_results_customAttributes
    from .versions_batch_get_post_response_results_entity_type import VersionsBatchGetPostResponse_results_entityType
    from .versions_batch_get_post_response_results_process_state import VersionsBatchGetPostResponse_results_processState

@dataclass
class VersionsBatchGetPostResponse_results(Parsable):
    # The approval status of the version. Only available when the review has been approved or rejected. For more information about the approval workflow, see the `Approval Workflows and Document Review <http://help.autodesk.com/view/BIM360D/ENU/?guid=GUID-2CC9A86E-2F4F-48EB-8EFA-FAA5FBECC20E>`_ documentation.
    approval_status: Optional[VersionsBatchGetPostResponse_results_approvalStatus] = None
    # The time and date the version was created.
    create_time: Optional[str] = None
    # The ID of the user who created the version.
    create_user_id: Optional[str] = None
    # The name of the user who created the version.
    create_user_name: Optional[str] = None
    # The list of custom attributes for each document. For more information about custom attributes, see the `Customize Documents with Attributes <http://help.autodesk.com/view/BIM360D/ENU/?guid=GUID-244FAB32-8E3B-4D1B-A1EB-C982DB93ABB3>`_ documentation.
    custom_attributes: Optional[list[VersionsBatchGetPostResponse_results_customAttributes]] = None
    # The type of version. Possible values:- ``SEED_FILE``: Documents that were not split into sheets when they were uploaded to BIM 360 Document Management.- ``DOCUMENT``: Documents that were split into sheets when they were uploaded to BIM 360 Document Management.
    entity_type: Optional[VersionsBatchGetPostResponse_results_entityType] = None
    # The ID of the related item.
    item_urn: Optional[str] = None
    # The time and date the version was last modified.
    last_modified_time: Optional[str] = None
    # The ID of the user who last modified the version.
    last_modified_user_id: Optional[str] = None
    # The name of the user who last modified the version.
    last_modified_user_name: Optional[str] = None
    # The name of the version. This corresponds to the file name in BIM 360 Document Management.
    name: Optional[str] = None
    # The sheet number. This is only relevant for documents uploaded to the Plans folder that were split into sheets.
    number: Optional[str] = None
    # The process state of the version.Possible values: ``NEEDS_PROCESSING``, ``PROCESSING``, ``PREVIOUS_SEED_PENDING``, ``EXTRACTION_PENDING``, ``SPLITTING``, ``PREVIOUS_DOC_PENDING``, ``PROCESSING_ABORTING``, ``PROCESSING_ABORTED``, ``PROCESSING_COMPLETE``, ``PROCESSING_PROMOTION``, ``PROCESSING_COPY``, ``PROCESSING_PROMOTING``, ``PROCESSING_COPYING``, ``PROCESSING_SUSPEND``
    process_state: Optional[VersionsBatchGetPostResponse_results_processState] = None
    # The revision number of the version. The revision number increases when you completely replace the version. For example when you reupload a document and overwrite the current document. Note that this is not the same as the version number, which increases when you update the document. For example, when you save the document. The revision number corresponds to the version in BIM 360 Document Management.
    revision_number: Optional[int] = None
    # The file size of the version's storage object, in bytes. This is only relevant for documents that were not split into sheets when they were uploaded to BIM 360 Document Management.
    storage_size: Optional[int] = None
    # The ID of the version's storage object. This is only relevant for documents that were not split into sheets when they were uploaded to BIM 360 Document Management.
    storage_urn: Optional[str] = None
    # The title of the version.
    title: Optional[str] = None
    # The ID of the version.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VersionsBatchGetPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VersionsBatchGetPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VersionsBatchGetPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .versions_batch_get_post_response_results_approval_status import VersionsBatchGetPostResponse_results_approvalStatus
        from .versions_batch_get_post_response_results_custom_attributes import VersionsBatchGetPostResponse_results_customAttributes
        from .versions_batch_get_post_response_results_entity_type import VersionsBatchGetPostResponse_results_entityType
        from .versions_batch_get_post_response_results_process_state import VersionsBatchGetPostResponse_results_processState

        from .versions_batch_get_post_response_results_approval_status import VersionsBatchGetPostResponse_results_approvalStatus
        from .versions_batch_get_post_response_results_custom_attributes import VersionsBatchGetPostResponse_results_customAttributes
        from .versions_batch_get_post_response_results_entity_type import VersionsBatchGetPostResponse_results_entityType
        from .versions_batch_get_post_response_results_process_state import VersionsBatchGetPostResponse_results_processState

        fields: dict[str, Callable[[Any], None]] = {
            "approvalStatus": lambda n : setattr(self, 'approval_status', n.get_object_value(VersionsBatchGetPostResponse_results_approvalStatus)),
            "createTime": lambda n : setattr(self, 'create_time', n.get_str_value()),
            "createUserId": lambda n : setattr(self, 'create_user_id', n.get_str_value()),
            "createUserName": lambda n : setattr(self, 'create_user_name', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(VersionsBatchGetPostResponse_results_customAttributes)),
            "entityType": lambda n : setattr(self, 'entity_type', n.get_enum_value(VersionsBatchGetPostResponse_results_entityType)),
            "itemUrn": lambda n : setattr(self, 'item_urn', n.get_str_value()),
            "lastModifiedTime": lambda n : setattr(self, 'last_modified_time', n.get_str_value()),
            "lastModifiedUserId": lambda n : setattr(self, 'last_modified_user_id', n.get_str_value()),
            "lastModifiedUserName": lambda n : setattr(self, 'last_modified_user_name', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "processState": lambda n : setattr(self, 'process_state', n.get_enum_value(VersionsBatchGetPostResponse_results_processState)),
            "revisionNumber": lambda n : setattr(self, 'revision_number', n.get_int_value()),
            "storageSize": lambda n : setattr(self, 'storage_size', n.get_int_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("approvalStatus", self.approval_status)
        writer.write_str_value("createTime", self.create_time)
        writer.write_str_value("createUserId", self.create_user_id)
        writer.write_str_value("createUserName", self.create_user_name)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_enum_value("entityType", self.entity_type)
        writer.write_str_value("itemUrn", self.item_urn)
        writer.write_str_value("lastModifiedTime", self.last_modified_time)
        writer.write_str_value("lastModifiedUserId", self.last_modified_user_id)
        writer.write_str_value("lastModifiedUserName", self.last_modified_user_name)
        writer.write_str_value("name", self.name)
        writer.write_str_value("number", self.number)
        writer.write_enum_value("processState", self.process_state)
        writer.write_int_value("revisionNumber", self.revision_number)
        writer.write_int_value("storageSize", self.storage_size)
        writer.write_str_value("storageUrn", self.storage_urn)
        writer.write_str_value("title", self.title)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ManifestGetResponse_stats(Parsable):
    # The number of design elements added between the compared model versions.
    added_objects: Optional[int] = None
    # The number of design elements which have update property values or altered geometry.
    changed_objects: Optional[int] = None
    # The compressed file size in bytes of the index. This file can be downloaded via the diff properties endpoint.
    content_length: Optional[int] = None
    # The number of design elements removed between the compared model versions.
    removed_objects: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManifestGetResponse_stats:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManifestGetResponse_stats
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ManifestGetResponse_stats()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "addedObjects": lambda n : setattr(self, 'added_objects', n.get_int_value()),
            "changedObjects": lambda n : setattr(self, 'changed_objects', n.get_int_value()),
            "contentLength": lambda n : setattr(self, 'content_length', n.get_int_value()),
            "removedObjects": lambda n : setattr(self, 'removed_objects', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("addedObjects", self.added_objects)
        writer.write_int_value("changedObjects", self.changed_objects)
        writer.write_int_value("contentLength", self.content_length)
        writer.write_int_value("removedObjects", self.removed_objects)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .review_sheets.review_sheets_request_builder import ReviewSheetsRequestBuilder
    from .review_sheets_publish.review_sheets_publish_request_builder import ReviewSheetsPublishRequestBuilder
    from .thumbnails_batch_get.thumbnails_batch_get_request_builder import ThumbnailsBatchGetRequestBuilder
    from .with_upload_get_response import WithUploadGetResponse

class WithUploadItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/sheets/v1/projects/{projectId}/uploads/{uploadId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithUploadItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/sheets/v1/projects/{projectId}/uploads/{uploadId}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WithUploadGetResponse]:
        """
        Checks the processing status of a specific uploaded file.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WithUploadGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .with_upload_get_response import WithUploadGetResponse

        return await self.request_adapter.send_async(request_info, WithUploadGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Checks the processing status of a specific uploaded file.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> WithUploadItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithUploadItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithUploadItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def review_sheets(self) -> ReviewSheetsRequestBuilder:
        """
        The reviewSheets property
        """
        from .review_sheets.review_sheets_request_builder import ReviewSheetsRequestBuilder

        return ReviewSheetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def review_sheets_publish(self) -> ReviewSheetsPublishRequestBuilder:
        """
        The reviewSheetsPublish property
        """
        from .review_sheets_publish.review_sheets_publish_request_builder import ReviewSheetsPublishRequestBuilder

        return ReviewSheetsPublishRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def thumbnails_batch_get(self) -> ThumbnailsBatchGetRequestBuilder:
        """
        The thumbnailsBatchGet property
        """
        from .thumbnails_batch_get.thumbnails_batch_get_request_builder import ThumbnailsBatchGetRequestBuilder

        return ThumbnailsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WithUploadItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .latest_get_response_document_versions_document_lineage import LatestGetResponse_documentVersions_documentLineage
    from .latest_get_response_document_versions_document_status import LatestGetResponse_documentVersions_documentStatus
    from .latest_get_response_document_versions_forge_type import LatestGetResponse_documentVersions_forgeType

@dataclass
class LatestGetResponse_documentVersions(Parsable):
    # The URN of the Model Derivative bubble for the document version.
    bubble_urn: Optional[str] = None
    # The date and time that the document version was created.
    create_time: Optional[datetime.datetime] = None
    # The unique identifier of the user who created the document version.
    create_user_id: Optional[str] = None
    # The display name of the document version.
    display_name: Optional[str] = None
    # A document from a model set.
    document_lineage: Optional[LatestGetResponse_documentVersions_documentLineage] = None
    # The status of the document. Possible values: ``Succeeded``, ``Failed``, ``Running``, ``Skipped``.
    document_status: Optional[LatestGetResponse_documentVersions_documentStatus] = None
    # The forge type associated with this document (used by the Document Management APIs). Possible values: ``versions:autodesk.bim360:Document``, ``versions:autodesk.bim360:File``.
    forge_type: Optional[LatestGetResponse_documentVersions_forgeType] = None
    # The name of the seed file version from which this document was originally extracted.
    original_seed_file_version_name: Optional[str] = None
    # The URN of the seed file version from which this document was originally extracted.
    original_seed_file_version_urn: Optional[str] = None
    # The URN of the document version.
    version_urn: Optional[str] = None
    # The ID of the geometry node in the derivative manifest to which this document version refers.
    viewable_guid: Optional[str] = None
    # The ID of the viewable for the document version.
    viewable_id: Optional[str] = None
    # The mime type of the viewable for the document version.
    viewable_mime: Optional[str] = None
    # The name of the viewable in the Model Derivative manifest.
    viewable_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> LatestGetResponse_documentVersions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: LatestGetResponse_documentVersions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return LatestGetResponse_documentVersions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .latest_get_response_document_versions_document_lineage import LatestGetResponse_documentVersions_documentLineage
        from .latest_get_response_document_versions_document_status import LatestGetResponse_documentVersions_documentStatus
        from .latest_get_response_document_versions_forge_type import LatestGetResponse_documentVersions_forgeType

        from .latest_get_response_document_versions_document_lineage import LatestGetResponse_documentVersions_documentLineage
        from .latest_get_response_document_versions_document_status import LatestGetResponse_documentVersions_documentStatus
        from .latest_get_response_document_versions_forge_type import LatestGetResponse_documentVersions_forgeType

        fields: dict[str, Callable[[Any], None]] = {
            "bubbleUrn": lambda n : setattr(self, 'bubble_urn', n.get_str_value()),
            "createTime": lambda n : setattr(self, 'create_time', n.get_datetime_value()),
            "createUserId": lambda n : setattr(self, 'create_user_id', n.get_str_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "documentLineage": lambda n : setattr(self, 'document_lineage', n.get_object_value(LatestGetResponse_documentVersions_documentLineage)),
            "documentStatus": lambda n : setattr(self, 'document_status', n.get_enum_value(LatestGetResponse_documentVersions_documentStatus)),
            "forgeType": lambda n : setattr(self, 'forge_type', n.get_enum_value(LatestGetResponse_documentVersions_forgeType)),
            "originalSeedFileVersionName": lambda n : setattr(self, 'original_seed_file_version_name', n.get_str_value()),
            "originalSeedFileVersionUrn": lambda n : setattr(self, 'original_seed_file_version_urn', n.get_str_value()),
            "versionUrn": lambda n : setattr(self, 'version_urn', n.get_str_value()),
            "viewableGuid": lambda n : setattr(self, 'viewable_guid', n.get_str_value()),
            "viewableId": lambda n : setattr(self, 'viewable_id', n.get_str_value()),
            "viewableMime": lambda n : setattr(self, 'viewable_mime', n.get_str_value()),
            "viewableName": lambda n : setattr(self, 'viewable_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("bubbleUrn", self.bubble_urn)
        writer.write_datetime_value("createTime", self.create_time)
        writer.write_str_value("createUserId", self.create_user_id)
        writer.write_str_value("displayName", self.display_name)
        writer.write_object_value("documentLineage", self.document_lineage)
        writer.write_enum_value("documentStatus", self.document_status)
        writer.write_enum_value("forgeType", self.forge_type)
        writer.write_str_value("originalSeedFileVersionName", self.original_seed_file_version_name)
        writer.write_str_value("originalSeedFileVersionUrn", self.original_seed_file_version_urn)
        writer.write_str_value("versionUrn", self.version_urn)
        writer.write_str_value("viewableGuid", self.viewable_guid)
        writer.write_str_value("viewableId", self.viewable_id)
        writer.write_str_value("viewableMime", self.viewable_mime)
        writer.write_str_value("viewableName", self.viewable_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .item.with_item_item_request_builder import WithItemItemRequestBuilder
    from .items_get_response import ItemsGetResponse
    from .items_post_request_body import ItemsPostRequestBody
    from .items_post_response import ItemsPostResponse
    from .with_item_id_transition.with_item_id_transition_request_builder import WithItemIdTransitionRequestBuilder

class ItemsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/items
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ItemsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}/items{?filter%5BballInCourtCompanies%5D*,filter%5BballInCourtRoles%5D*,filter%5BballInCourtUsers%5D*,filter%5BcreatedAt%5D*,filter%5BcreatedBy%5D*,filter%5BdueDate%5D*,filter%5Bid%5D*,filter%5Bidentifier%5D*,filter%5BleadTime%5D*,filter%5BmanagerDueDate%5D*,filter%5BmanagerType%5D*,filter%5Bmanager%5D*,filter%5BpackageId%5D*,filter%5BpublishedDate%5D*,filter%5BreceivedFromReview%5D*,filter%5BreceivedFromSubmitter%5D*,filter%5BrequiredApprovalDate%5D*,filter%5BrequiredDate%5D*,filter%5BrequiredOnJobDate%5D*,filter%5BresponseId%5D*,filter%5BreviewResponseId%5D*,filter%5Brevision%5D*,filter%5BsentToReview%5D*,filter%5BsentToSubmitter%5D*,filter%5BspecId%5D*,filter%5BstateId%5D*,filter%5BstatusId%5D*,filter%5BsubcontractorType%5D*,filter%5Bsubcontractor%5D*,filter%5BsubmitterDueDate%5D*,filter%5Btitle%5D*,filter%5BtypeId%5D*,filter%5BupdatedAt%5D*,filter%5Bwatchers%5D*,limit*,offset*,search*,sort*}", path_parameters)
    
    def by_item_id(self,item_id: str) -> WithItemItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.submittals.v2.projects.item.items.item collection
        param item_id: The ID of the submittal item. To find the item ID, call `GET items </en/docs/acc/v1/reference/http/submittals-items-GET/>`_.
        Returns: WithItemItemRequestBuilder
        """
        if item_id is None:
            raise TypeError("item_id cannot be null.")
        from .item.with_item_item_request_builder import WithItemItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["itemId"] = item_id
        return WithItemItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ItemsRequestBuilderGetQueryParameters]] = None) -> Optional[ItemsGetResponse]:
        """
        Retrieves information about all the submittal items in a project that the user has permission to view.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ItemsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .items_get_response import ItemsGetResponse

        return await self.request_adapter.send_async(request_info, ItemsGetResponse, None)
    
    async def post(self,body: ItemsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[ItemsPostResponse]:
        """
        Creates a new submittal item in the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ItemsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .items_post_response import ItemsPostResponse

        return await self.request_adapter.send_async(request_info, ItemsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ItemsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves information about all the submittal items in a project that the user has permission to view.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: ItemsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new submittal item in the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_item_id_transition(self,item_id: str) -> WithItemIdTransitionRequestBuilder:
        """
        Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/items/{itemId}:transition
        param item_id: The ID of the submittal item. To find the item ID, call `GET items </en/docs/acc/v1/reference/http/submittals-items-GET/>`_.
        Returns: WithItemIdTransitionRequestBuilder
        """
        if item_id is None:
            raise TypeError("item_id cannot be null.")
        from .with_item_id_transition.with_item_id_transition_request_builder import WithItemIdTransitionRequestBuilder

        return WithItemIdTransitionRequestBuilder(self.request_adapter, self.path_parameters, item_id)
    
    def with_url(self,raw_url: str) -> ItemsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ItemsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ItemsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ItemsRequestBuilderGetQueryParameters():
        """
        Retrieves information about all the submittal items in a project that the user has permission to view.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterball_in_court_companies":
                return "filter%5BballInCourtCompanies%5D"
            if original_name == "filterball_in_court_roles":
                return "filter%5BballInCourtRoles%5D"
            if original_name == "filterball_in_court_users":
                return "filter%5BballInCourtUsers%5D"
            if original_name == "filtercreated_at":
                return "filter%5BcreatedAt%5D"
            if original_name == "filtercreated_by":
                return "filter%5BcreatedBy%5D"
            if original_name == "filterdue_date":
                return "filter%5BdueDate%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filteridentifier":
                return "filter%5Bidentifier%5D"
            if original_name == "filterlead_time":
                return "filter%5BleadTime%5D"
            if original_name == "filtermanager":
                return "filter%5Bmanager%5D"
            if original_name == "filtermanager_due_date":
                return "filter%5BmanagerDueDate%5D"
            if original_name == "filtermanager_type":
                return "filter%5BmanagerType%5D"
            if original_name == "filterpackage_id":
                return "filter%5BpackageId%5D"
            if original_name == "filterpublished_date":
                return "filter%5BpublishedDate%5D"
            if original_name == "filterreceived_from_review":
                return "filter%5BreceivedFromReview%5D"
            if original_name == "filterreceived_from_submitter":
                return "filter%5BreceivedFromSubmitter%5D"
            if original_name == "filterrequired_approval_date":
                return "filter%5BrequiredApprovalDate%5D"
            if original_name == "filterrequired_date":
                return "filter%5BrequiredDate%5D"
            if original_name == "filterrequired_on_job_date":
                return "filter%5BrequiredOnJobDate%5D"
            if original_name == "filterresponse_id":
                return "filter%5BresponseId%5D"
            if original_name == "filterreview_response_id":
                return "filter%5BreviewResponseId%5D"
            if original_name == "filterrevision":
                return "filter%5Brevision%5D"
            if original_name == "filtersent_to_review":
                return "filter%5BsentToReview%5D"
            if original_name == "filtersent_to_submitter":
                return "filter%5BsentToSubmitter%5D"
            if original_name == "filterspec_id":
                return "filter%5BspecId%5D"
            if original_name == "filterstate_id":
                return "filter%5BstateId%5D"
            if original_name == "filterstatus_id":
                return "filter%5BstatusId%5D"
            if original_name == "filtersubcontractor":
                return "filter%5Bsubcontractor%5D"
            if original_name == "filtersubcontractor_type":
                return "filter%5BsubcontractorType%5D"
            if original_name == "filtersubmitter_due_date":
                return "filter%5BsubmitterDueDate%5D"
            if original_name == "filtertitle":
                return "filter%5Btitle%5D"
            if original_name == "filtertype_id":
                return "filter%5BtypeId%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filterwatchers":
                return "filter%5Bwatchers%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "search":
                return "search"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter items associated with the specified ball-in-court company ID. You can specify multiple companies. Separate multiple values with commas. For example, ``filter[ballInCourtCompanies]=WD43ZJGKDFLFH``.
        filterball_in_court_companies: Optional[str] = None

        # Filter items associated with the specified ball-in-court role ID. You can specify multiple roles. Separate multiple values with commas. For example, ``filter[ballInCourtRoles]=WD43ZJGKDFLFH``.
        filterball_in_court_roles: Optional[str] = None

        # Filter items associated with the specified ball-in-court user ID. You can specify multiple users. Separate multiple values with commas. For example, ``filter[ballInCourtUsers]=HNRCQ6JTWAED``.
        filterball_in_court_users: Optional[str] = None

        # Filter items with the specified creation date, using the following URL-encoded format YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtercreated_at: Optional[str] = None

        # Filter items that were created by the specified user by specifying the user’s Autodesk ID. For example, ``filter[createdBy]=PER8KQPK2JRT``. To find the ID call `GET users <https://aps.autodesk.com/en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
        filtercreated_by: Optional[str] = None

        # Filter items with the specified due date, using the following URL-encoded format YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterdue_date: Optional[str] = None

        # Filter submittal items by their unique internal submittal item ID (UUID). You can specify multiple values. Separate multiple values with commas. For example ``filter[id]=b8cc9324-6759-4f07-8ce3-725d5afd4f11``.
        filterid: Optional[str] = None

        # Filter items with the specified submittal item ID (the submittal item ID in the UI). You can specify multiple values. Separate multiple values with commas. For example, ``filter[identifier]=2``.
        filteridentifier: Optional[str] = None

        # Filter items with the specified lead time. You can specify multiple values. Separate multiple values with commas. For example, ``filter[leadTime]=100``.
        filterlead_time: Optional[str] = None

        # Filter items with the specified manager Autodesk ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[manager]=WD43ZJGKDFLFH``.
        filtermanager: Optional[str] = None

        # Filter items with the specified manager's due date, using the following URL-encoded format YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtermanager_due_date: Optional[str] = None

        # Filter items with the specified manager type. You can specify multiple values. Separate multiple values with commas. For example, ``filter[managerType]=1``.
        filtermanager_type: Optional[str] = None

        # Filter items with the specified package ID. You can specify multiple values. Separate multiple values with commas.For example, ``filter[packageId]=06fa0c1b-6462-459d-8a38-0aff11bfe868``.In order to filter items with no package, use 'noPackage' value.
        filterpackage_id: Optional[str] = None

        # Filter items based on their published date using the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterpublished_date: Optional[str] = None

        # Filter items based on the date they were moved forward by the reviewer using the URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterreceived_from_review: Optional[str] = None

        # Filter items based on the date they were received from the submitter. Use the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterreceived_from_submitter: Optional[str] = None

        # Filter items based on their required approval date using the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterrequired_approval_date: Optional[str] = None

        # Filter items based on the date the responsible contractor needs to submit the submittal to the submittal manager, using the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterrequired_date: Optional[str] = None

        # Filter items based on their required on-job (jobsite) date using the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterrequired_on_job_date: Optional[str] = None

        # Filter items with the specified final response ID. You can specify multiple IDs. You can specify multiple values. Separate multiple values with commas. For example, ``filter[responseId]=1w66d30b-7dc1-4a65-991d-d739a1381rf4``.
        filterresponse_id: Optional[str] = None

        # Filter items with the specified review response ID. You can specify multiple IDs. You can specify multiple values. Separate multiple values with commas. For example, ``filter[reviewResponseId]=1w66d30b-7dc1-4a65-991d-d739a1381rf4``.
        filterreview_response_id: Optional[str] = None

        # Filter items with the specified revision number. You can specify multiple values. Separate multiple values with commas. For example, ``filter[revision]=1``.
        filterrevision: Optional[str] = None

        # Filter items based on the date they were sent for review, in the URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtersent_to_review: Optional[str] = None

        # Filter items based on the date they were sent to the submitter. Use the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtersent_to_submitter: Optional[str] = None

        # Filter items with the specified spec ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[specId]=b4aa3864-5706-4a7b-b06c-a792e8b2df23``
        filterspec_id: Optional[str] = None

        # Filter items with the specified state ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[stateId]=rev``.
        filterstate_id: Optional[str] = None

        # Filter items with the specified status ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[statusId]=1``
        filterstatus_id: Optional[str] = None

        # Filter items with the specified subcontractor Autodesk ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[subcontractor]=WD43ZJGKDFLFH``.
        filtersubcontractor: Optional[str] = None

        # Filter items with the specified subcontractor type. You can specify multiple values. Separate multiple values with commas. For example, ``filter[subcontractorType]=1``.
        filtersubcontractor_type: Optional[str] = None

        # Filter items with the specified submitter's due date, using the following URL-encoded format YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtersubmitter_due_date: Optional[str] = None

        # Filter items with the specified title. You can specify multiple values. Separate multiple values with commas. For example, ``filter[title]=``Shop Drawings````.
        filtertitle: Optional[str] = None

        # Filter items with the specified type ID. You can specify multiple values. Separate multiple values with commas. For example, ``filter[typeId]=06fa0c1b-6462-459d-8a38-0aff11bfe868``.
        filtertype_id: Optional[str] = None

        # Filter items by their last updated date using the following URL-encoded format: YYYY-MM-DD. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterupdated_at: Optional[str] = None

        # Filter items that are associated with the specified watcher, by specifying the watcher’s Autodesk ID. For example, ``filter[watchers]=PER8KQPK2JRT``. You can specify more than one watcher. Separate multiple values with commas. To find the ID call `GET users <https://aps.autodesk.com/en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
        filterwatchers: Optional[str] = None

        # The maximum number of results per page. Possible values: ``1``- ``50``. Default value: ``20``. For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The number of results to skip before starting to return data. For example, to skip the first 20 results, include ``offset=20`` in the query string. For more details, see the `JSON API Paging Help documentation <https://jsonapi.org/format/#fetching-pagination>`_.
        offset: Optional[int] = None

        # Search for items by querying a specified string within specific fields (``identifier``, ``title``, ``specIdentifier``, ``ballInCourt``), and retrieve the associated items that match the search criteria. This includes items where the string matches part of a field. For example, ``search=1``.
        search: Optional[str] = None

        # Sort items by specified fields. Separate multiple values with commas. To sort in descending or ascending order, add ``desc`` or ``asc`` after the sort criteria. For example, ``statusId asc``.Possible values: ``ballInCourtUsers``, ``ballInCourtCompanies``, ``ballInCourtRoles``, ``ballInCourtType``, ``customIdentifier``, ``customIdentifierHumanReadable``, ``createdAt``, ``createdBy``, ``description``, ``dueDate``, ``id``, ``identifier``, ``leadTime``, ``manager``, ``managerType``, ``managerDueDate``, ``package.identifier``, ``package.spec.identifier``, ``packageId``, ``packageIdentifier``, ``packageSpecIdentifier``, ``packageTitle``, ``priority``, ``publishedBy``, ``publishedDate``, ``receivedFromReview``, ``receivedFromSubmitter``, ``requiredApprovalDate``, ``requiredDate``, ``requiredOnJobDate``, ``respondedAt``, ``respondedBy``, ``response.value``, ``responseComment``, ``responseId``, ``revision``, ``sentToReview``, ``sentToReviewBy``, ``sentToSubmitter``, ``spec.identifier``, ``specId``, ``specIdentifier``, ``specTitle``, ``stateId``, ``statusId``, ``subcontractor``, ``subcontractorType``, ``submittedBy``, ``submitterDueDate``, ``subsection``, ``title``, ``type.value``, ``typeId``, ``updatedAt``, ``updatedBy``, ``watchers``.
        sort: Optional[str] = None

    
    @dataclass
    class ItemsRequestBuilderGetRequestConfiguration(RequestConfiguration[ItemsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ItemsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class ClassificationSystemsGetResponse_results_type(str, Enum):
    CLASSIFICATION_SYSTEM_1 = "CLASSIFICATION_SYSTEM_1",
    CLASSIFICATION_SYSTEM_2 = "CLASSIFICATION_SYSTEM_2",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ItemsPostResponse_permittedActions_fields(Parsable):
    """
    A list of field names for which values must be provided when performing the action. An empty array indicates no specific set of values.
    """
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ItemsPostResponse_permittedActions_fields:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ItemsPostResponse_permittedActions_fields
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ItemsPostResponse_permittedActions_fields()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .metadata_get_response_region import MetadataGetResponse_region
    from .metadata_get_response_versions import MetadataGetResponse_versions

@dataclass
class MetadataGetResponse(Parsable):
    # The ACC project ID.
    project_id: Optional[str] = None
    # The region of the specification PDFs that were uploaded for this project. Currently, AutoSpecs supports CSI MasterFormat for the United States and Canada.Possible values: ``USA``, ``Canada``, ``Others``
    region: Optional[MetadataGetResponse_region] = None
    # Information about the versions for the project.
    versions: Optional[list[MetadataGetResponse_versions]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MetadataGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MetadataGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MetadataGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .metadata_get_response_region import MetadataGetResponse_region
        from .metadata_get_response_versions import MetadataGetResponse_versions

        from .metadata_get_response_region import MetadataGetResponse_region
        from .metadata_get_response_versions import MetadataGetResponse_versions

        fields: dict[str, Callable[[Any], None]] = {
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "region": lambda n : setattr(self, 'region', n.get_enum_value(MetadataGetResponse_region)),
            "versions": lambda n : setattr(self, 'versions', n.get_collection_of_object_values(MetadataGetResponse_versions)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("projectId", self.project_id)
        writer.write_enum_value("region", self.region)
        writer.write_collection_of_object_values("versions", self.versions)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_request_body_attachments_attachment_type import AttachmentsPostRequestBody_attachments_attachmentType

@dataclass
class AttachmentsPostRequestBody_attachments(Parsable):
    # The unique identifier for the attachment, set by the client when creating the attachment reference. This can be any unique GUID, but it is recommended to use the OSS storage GUID. For more information, see the `Upload Issue Attachment </en/docs/acc/v1/tutorials/issues/upload-issue-attachments/>`_ tutorial.
    attachment_id: Optional[UUID] = None
    # The type of attachment to create. Set to ``issue-attachment``. Will always be: ``issue-attachment``
    attachment_type: Optional[AttachmentsPostRequestBody_attachments_attachmentType] = None
    # The human-readable display name for the attachment, including the file extension (for example, ``.pdf``, ``.jpg``, ``.dwg``). This name appears in the ACC web UI and is used when downloading the file from the issue.
    display_name: Optional[str] = None
    # The unique filename of the attachment, typically formatted as {attachmentId}.{fileExtension}.This value must exactly match the name of the file stored in Autodesk Docs (OSS) that you uploaded via the OSS process.For more information, see the `Upload Issue Attachment </en/docs/acc/v1/tutorials/issues/upload-issue-attachments/>`_ tutorial.
    file_name: Optional[str] = None
    # The Object Storage Service (OSS) URN that uniquely identifies where the file is stored in Autodesk's cloud infrastructure. You obtain this value after uploading the file to OSS (see the `Upload Issue Attachment </en/docs/acc/v1/tutorials/issues/upload-issue-attachments/>`_ tutorial) or by retrieving it from an existing attachment (see the `Downloading Issue Attachments </en/docs/acc/v1/tutorials/issues/download-issue-attachments/>`_ tutorial).
    storage_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostRequestBody_attachments:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostRequestBody_attachments
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostRequestBody_attachments()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_request_body_attachments_attachment_type import AttachmentsPostRequestBody_attachments_attachmentType

        from .attachments_post_request_body_attachments_attachment_type import AttachmentsPostRequestBody_attachments_attachmentType

        fields: dict[str, Callable[[Any], None]] = {
            "attachmentId": lambda n : setattr(self, 'attachment_id', n.get_uuid_value()),
            "attachmentType": lambda n : setattr(self, 'attachment_type', n.get_enum_value(AttachmentsPostRequestBody_attachments_attachmentType)),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("attachmentId", self.attachment_id)
        writer.write_enum_value("attachmentType", self.attachment_type)
        writer.write_str_value("displayName", self.display_name)
        writer.write_str_value("fileName", self.file_name)
        writer.write_str_value("storageUrn", self.storage_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.with_segment_item_request_builder import WithSegmentItemRequestBuilder
    from .segments_get_response import SegmentsGetResponse
    from .segments_post_request_body import SegmentsPostRequestBody
    from .segments_post_response import SegmentsPostResponse

class SegmentsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/templates/{templateId}/segments
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new SegmentsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/templates/{templateId}/segments{?filter%5BlastModifiedSince%5D*,filter%5Bname%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_segment_id(self,segment_id: UUID) -> WithSegmentItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.templates.item.segments.item collection
        param segment_id: The segment ID. To obtain a segment ID, use `GET templates/:templateId/segments </en/docs/bim360/v1/reference/http/cost-segments-GET/>`_.
        Returns: WithSegmentItemRequestBuilder
        """
        if segment_id is None:
            raise TypeError("segment_id cannot be null.")
        from .item.with_segment_item_request_builder import WithSegmentItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["segmentId"] = segment_id
        return WithSegmentItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SegmentsRequestBuilderGetQueryParameters]] = None) -> Optional[SegmentsGetResponse]:
        """
        Retrieves all of the segments in a budget code template.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SegmentsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .segments_get_response import SegmentsGetResponse

        return await self.request_adapter.send_async(request_info, SegmentsGetResponse, None)
    
    async def post(self,body: SegmentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[SegmentsPostResponse]:
        """
        Creates a new segment in the budget code template.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SegmentsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .segments_post_response import SegmentsPostResponse

        return await self.request_adapter.send_async(request_info, SegmentsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SegmentsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all of the segments in a budget code template.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: SegmentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new segment in the budget code template.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> SegmentsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SegmentsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return SegmentsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class SegmentsRequestBuilderGetQueryParameters():
        """
        Retrieves all of the segments in a budget code template.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterlast_modified_since":
                return "filter%5BlastModifiedSince%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Returns only items that were modified since the specified date and time, in ISO 8601 format. For example, ``filter[lastModifiedSince]=2020-03-01T13:00:00Z``.
        filterlast_modified_since: Optional[str] = None

        # Returns only items with the specified name. For example, ``filter[name]=Labor``.Max length: 255
        filtername: Optional[str] = None

        # The maximum number of records returned per page. Default: ``100``. A page may contain fewer records than the limit if there are fewer matching items or if it is the last page of results.
        limit: Optional[int] = None

        # The number of records to skip before returning results. Used together with ``limit`` to paginate through results, where ``offset`` specifies the starting point and ``limit`` specifies the number of records to return.
        offset: Optional[int] = None

        # Defines the sort order for the results. Each attribute can be sorted in ``asc`` (default) or ``desc`` order. For example, ``sort=name desc`` sorts the results by name in descending order.
        sort: Optional[str] = None

    
    @dataclass
    class SegmentsRequestBuilderGetRequestConfiguration(RequestConfiguration[SegmentsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class SegmentsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_takeoff_type_get_response_model_mappings import WithTakeoffTypeGetResponse_modelMappings
    from .with_takeoff_type_get_response_primary_quantity_definition import WithTakeoffTypeGetResponse_primaryQuantityDefinition
    from .with_takeoff_type_get_response_property_definitions import WithTakeoffTypeGetResponse_propertyDefinitions
    from .with_takeoff_type_get_response_secondary_quantity_definitions import WithTakeoffTypeGetResponse_secondaryQuantityDefinitions
    from .with_takeoff_type_get_response_shape_type import WithTakeoffTypeGetResponse_shapeType
    from .with_takeoff_type_get_response_tool import WithTakeoffTypeGetResponse_tool

@dataclass
class WithTakeoffTypeGetResponse(Parsable):
    # The color of the takeoff type as it appears on the sheet or model.A color is represented by a value consisting of 6 or 8 hexadecimal digits, prefixed with a hash (#) symbol. In the 6-digit notation, the first pair of digits represent the red channel, the middle pair of digits represent the green channel and the last pair of digits represent the blue channel. The color is completely opaque.The 8-digit notation follows the structure of the 6-digit notation, where the additional last 2-digits represent the alpha channel (``00`` is transparent and ``ff`` is fully opaque).
    border_color: Optional[str] = None
    # The color of the takeoff type as it appears on the sheet or model.A color is represented by a value consisting of 6 or 8 hexadecimal digits, prefixed with a hash (#) symbol. In the 6-digit notation, the first pair of digits represent the red channel, the middle pair of digits represent the green channel and the last pair of digits represent the blue channel. The color is completely opaque.The 8-digit notation follows the structure of the 6-digit notation, where the additional last 2-digits represent the alpha channel (``00`` is transparent and ``ff`` is fully opaque).
    color: Optional[str] = None
    # The size of the count marker.You must set this value in the UI.
    count_marker_size: Optional[float] = None
    # The date and time when the resource was created, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    created_at: Optional[datetime.datetime] = None
    # A description of the takeoff type.Max length: 256
    description: Optional[str] = None
    # The takeoff type ID.
    id: Optional[UUID] = None
    # A list of model properties used to calculate output.Only relevant for 3D models.
    model_mappings: Optional[list[WithTakeoffTypeGetResponse_modelMappings]] = None
    # The takeoff type name.Max length: 64
    name: Optional[str] = None
    # The classification quantity details.
    primary_quantity_definition: Optional[WithTakeoffTypeGetResponse_primaryQuantityDefinition] = None
    # A list of additional, user provided parameters associated with a 2D takeoff type.
    property_definitions: Optional[list[WithTakeoffTypeGetResponse_propertyDefinitions]] = None
    # A list of secondary classification quantity details.
    secondary_quantity_definitions: Optional[list[WithTakeoffTypeGetResponse_secondaryQuantityDefinitions]] = None
    # The shape of the count marker.A count marker is one of the takeoff tool type options that is selected in the UI when creating a takeoff type.Possible values: ``CIRCLE``, ``TRIANGLE``, ``SQUARE``, ``DIAMOND``, ``CHECKMARK``.
    shape_type: Optional[WithTakeoffTypeGetResponse_shapeType] = None
    # The type of tool used to create takeoff items of this takeoff type.Possible values: ``COUNT``, ``DISTANCE``, ``AREA``, ``SELECT``.Corresponding UI names: ``COUNT``, ``LINEAR``, ``AREA``, ``BIM``.
    tool: Optional[WithTakeoffTypeGetResponse_tool] = None
    # The date and time when the resource was last updated, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    updated_at: Optional[datetime.datetime] = None
    # The name of the user who last updated the resource.
    updated_by_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTakeoffTypeGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTakeoffTypeGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTakeoffTypeGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_takeoff_type_get_response_model_mappings import WithTakeoffTypeGetResponse_modelMappings
        from .with_takeoff_type_get_response_primary_quantity_definition import WithTakeoffTypeGetResponse_primaryQuantityDefinition
        from .with_takeoff_type_get_response_property_definitions import WithTakeoffTypeGetResponse_propertyDefinitions
        from .with_takeoff_type_get_response_secondary_quantity_definitions import WithTakeoffTypeGetResponse_secondaryQuantityDefinitions
        from .with_takeoff_type_get_response_shape_type import WithTakeoffTypeGetResponse_shapeType
        from .with_takeoff_type_get_response_tool import WithTakeoffTypeGetResponse_tool

        from .with_takeoff_type_get_response_model_mappings import WithTakeoffTypeGetResponse_modelMappings
        from .with_takeoff_type_get_response_primary_quantity_definition import WithTakeoffTypeGetResponse_primaryQuantityDefinition
        from .with_takeoff_type_get_response_property_definitions import WithTakeoffTypeGetResponse_propertyDefinitions
        from .with_takeoff_type_get_response_secondary_quantity_definitions import WithTakeoffTypeGetResponse_secondaryQuantityDefinitions
        from .with_takeoff_type_get_response_shape_type import WithTakeoffTypeGetResponse_shapeType
        from .with_takeoff_type_get_response_tool import WithTakeoffTypeGetResponse_tool

        fields: dict[str, Callable[[Any], None]] = {
            "borderColor": lambda n : setattr(self, 'border_color', n.get_str_value()),
            "color": lambda n : setattr(self, 'color', n.get_str_value()),
            "countMarkerSize": lambda n : setattr(self, 'count_marker_size', n.get_float_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "modelMappings": lambda n : setattr(self, 'model_mappings', n.get_collection_of_object_values(WithTakeoffTypeGetResponse_modelMappings)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "primaryQuantityDefinition": lambda n : setattr(self, 'primary_quantity_definition', n.get_object_value(WithTakeoffTypeGetResponse_primaryQuantityDefinition)),
            "propertyDefinitions": lambda n : setattr(self, 'property_definitions', n.get_collection_of_object_values(WithTakeoffTypeGetResponse_propertyDefinitions)),
            "secondaryQuantityDefinitions": lambda n : setattr(self, 'secondary_quantity_definitions', n.get_collection_of_object_values(WithTakeoffTypeGetResponse_secondaryQuantityDefinitions)),
            "shapeType": lambda n : setattr(self, 'shape_type', n.get_enum_value(WithTakeoffTypeGetResponse_shapeType)),
            "tool": lambda n : setattr(self, 'tool', n.get_enum_value(WithTakeoffTypeGetResponse_tool)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("borderColor", self.border_color)
        writer.write_str_value("color", self.color)
        writer.write_float_value("countMarkerSize", self.count_marker_size)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_collection_of_object_values("modelMappings", self.model_mappings)
        writer.write_str_value("name", self.name)
        writer.write_object_value("primaryQuantityDefinition", self.primary_quantity_definition)
        writer.write_collection_of_object_values("propertyDefinitions", self.property_definitions)
        writer.write_collection_of_object_values("secondaryQuantityDefinitions", self.secondary_quantity_definitions)
        writer.write_enum_value("shapeType", self.shape_type)
        writer.write_enum_value("tool", self.tool)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedByName", self.updated_by_name)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .latest_get_response_document_versions import LatestGetResponse_documentVersions
    from .latest_get_response_status import LatestGetResponse_status

@dataclass
class LatestGetResponse(Parsable):
    # The date and time that the model set version was created.
    create_time: Optional[datetime.datetime] = None
    # The document versions included in this version of the model set.
    document_versions: Optional[list[LatestGetResponse_documentVersions]] = None
    # The GUID that uniquely identifies the model set.
    model_set_id: Optional[UUID] = None
    # The creation status of the model set version. Possible values: ``Pending``, ``Processing``, ``Successful``, ``Partial``, ``Failed``.
    status: Optional[LatestGetResponse_status] = None
    # The model set version number.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> LatestGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: LatestGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return LatestGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .latest_get_response_document_versions import LatestGetResponse_documentVersions
        from .latest_get_response_status import LatestGetResponse_status

        from .latest_get_response_document_versions import LatestGetResponse_documentVersions
        from .latest_get_response_status import LatestGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "createTime": lambda n : setattr(self, 'create_time', n.get_datetime_value()),
            "documentVersions": lambda n : setattr(self, 'document_versions', n.get_collection_of_object_values(LatestGetResponse_documentVersions)),
            "modelSetId": lambda n : setattr(self, 'model_set_id', n.get_uuid_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(LatestGetResponse_status)),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createTime", self.create_time)
        writer.write_collection_of_object_values("documentVersions", self.document_versions)
        writer.write_uuid_value("modelSetId", self.model_set_id)
        writer.write_enum_value("status", self.status)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.naming_standards_item_request_builder import NamingStandardsItemRequestBuilder

class NamingStandardsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects/{project-id}/naming-standards
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new NamingStandardsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects/{project%2Did}/naming-standards", path_parameters)
    
    def by_id(self,id: UUID) -> NamingStandardsItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.bim360.docs.v1.projects.item.namingStandards.item collection
        param id: The ID of the file naming standard.The file naming standard is applied to the project files folder or its subfolders. To find the ID:- For the project files folder, call `GET hubs/:hub_id/projects/:project_id/topFolders </en/docs/data/v2/reference/http/hubs-hub_id-projects-project_id-topFolders-GET/>`_.- For subfolders, call `GET projects/:project_id/folders/:folder_id/contents </en/docs/data/v2/reference/http/projects-project_id-folders-folder_id-contents-GET/>`_.- For a specific folder, call `GET projects/:project_id/folders/:folder_id </en/docs/data/v2/reference/http/projects-project_id-folders-folder_id-GET/>`_.The ID is under ``data.attributes.extension.data.namingStandardIds``.
        Returns: NamingStandardsItemRequestBuilder
        """
        if id is None:
            raise TypeError("id cannot be null.")
        from .item.naming_standards_item_request_builder import NamingStandardsItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["id"] = id
        return NamingStandardsItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .manifest_get_response_errors import ManifestGetResponse_errors
    from .manifest_get_response_schema import ManifestGetResponse_schema
    from .manifest_get_response_seed_files import ManifestGetResponse_seedFiles
    from .manifest_get_response_stats import ManifestGetResponse_stats
    from .manifest_get_response_status import ManifestGetResponse_status

@dataclass
class ManifestGetResponse(Parsable):
    # creation timestamp.
    created_at: Optional[datetime.datetime] = None
    # errors.
    errors: Optional[list[ManifestGetResponse_errors]] = None
    # project ID.
    project_id: Optional[UUID] = None
    # current schema version.Possible values: ``2.0.0``
    schema: Optional[ManifestGetResponse_schema] = None
    # seed files.
    seed_files: Optional[list[ManifestGetResponse_seedFiles]] = None
    # High level statistics about the diff index
    stats: Optional[list[ManifestGetResponse_stats]] = None
    # manifest status.Possible values: ``Failed``, ``Running``, ``Succeeded``
    status: Optional[ManifestGetResponse_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManifestGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManifestGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ManifestGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .manifest_get_response_errors import ManifestGetResponse_errors
        from .manifest_get_response_schema import ManifestGetResponse_schema
        from .manifest_get_response_seed_files import ManifestGetResponse_seedFiles
        from .manifest_get_response_stats import ManifestGetResponse_stats
        from .manifest_get_response_status import ManifestGetResponse_status

        from .manifest_get_response_errors import ManifestGetResponse_errors
        from .manifest_get_response_schema import ManifestGetResponse_schema
        from .manifest_get_response_seed_files import ManifestGetResponse_seedFiles
        from .manifest_get_response_stats import ManifestGetResponse_stats
        from .manifest_get_response_status import ManifestGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "errors": lambda n : setattr(self, 'errors', n.get_collection_of_object_values(ManifestGetResponse_errors)),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "schema": lambda n : setattr(self, 'schema', n.get_enum_value(ManifestGetResponse_schema)),
            "seedFiles": lambda n : setattr(self, 'seed_files', n.get_collection_of_object_values(ManifestGetResponse_seedFiles)),
            "stats": lambda n : setattr(self, 'stats', n.get_collection_of_object_values(ManifestGetResponse_stats)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ManifestGetResponse_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_collection_of_object_values("errors", self.errors)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_enum_value("schema", self.schema)
        writer.write_collection_of_object_values("seedFiles", self.seed_files)
        writer.write_collection_of_object_values("stats", self.stats)
        writer.write_enum_value("status", self.status)
    




from enum import Enum

class WithProjectGetResponse_products_key(str, Enum):
    FilterKey = "filter[key]",
    AutoSpecs = "autoSpecs",
    Build = "build",
    Cost = "cost",
    DesignCollaboration = "designCollaboration",
    Docs = "docs",
    Insight = "insight",
    ModelCoordination = "modelCoordination",
    ProjectAdministration = "projectAdministration",
    Takeoff = "takeoff",
    Assets = "assets",
    CostManagement = "costManagement",
    DocumentManagement = "documentManagement",
    Field = "field",
    FieldManagement = "fieldManagement",
    Glue = "glue",
    Plan = "plan",
    ProjectHome = "projectHome",
    ProjectManagement = "projectManagement",
    Quantification = "quantification",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class PhotosFilterPostRequestBody_filter(Parsable):
    # The createdAt property
    created_at: Optional[str] = None
    # The createdBy property
    created_by: Optional[list[str]] = None
    # The id property
    id: Optional[list[str]] = None
    # The mediaType property
    media_type: Optional[list[str]] = None
    # The takenAt property
    taken_at: Optional[str] = None
    # The title property
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PhotosFilterPostRequestBody_filter:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PhotosFilterPostRequestBody_filter
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PhotosFilterPostRequestBody_filter()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_collection_of_primitive_values(str)),
            "id": lambda n : setattr(self, 'id', n.get_collection_of_primitive_values(str)),
            "mediaType": lambda n : setattr(self, 'media_type', n.get_collection_of_primitive_values(str)),
            "takenAt": lambda n : setattr(self, 'taken_at', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdAt", self.created_at)
        writer.write_collection_of_primitive_values("createdBy", self.created_by)
        writer.write_collection_of_primitive_values("id", self.id)
        writer.write_collection_of_primitive_values("mediaType", self.media_type)
        writer.write_str_value("takenAt", self.taken_at)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .approval_statuses_get_response_results_review_status import ApprovalStatusesGetResponse_results_review_status

@dataclass
class ApprovalStatusesGetResponse_results_review(Parsable):
    """
    Metadata about the review in which this file version was included.
    """
    # The ID of the review.
    id: Optional[UUID] = None
    # A unique, auto-incrementing number assigned to the review when it is first submitted.This ID does not change, even if the review is sent back to the initiator and goes through multiple rounds.It identifies the review within the project and reflects the order in which reviews were created.
    sequence_id: Optional[int] = None
    # The current status of the review.Possible values: ``OPEN``, ``CLOSED``, ``VOID``, ``FAILED``.
    status: Optional[ApprovalStatusesGetResponse_results_review_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ApprovalStatusesGetResponse_results_review:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ApprovalStatusesGetResponse_results_review
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ApprovalStatusesGetResponse_results_review()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .approval_statuses_get_response_results_review_status import ApprovalStatusesGetResponse_results_review_status

        from .approval_statuses_get_response_results_review_status import ApprovalStatusesGetResponse_results_review_status

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "sequenceId": lambda n : setattr(self, 'sequence_id', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ApprovalStatusesGetResponse_results_review_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("id", self.id)
        writer.write_int_value("sequenceId", self.sequence_id)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .workflows_post_request_body_steps_candidates import WorkflowsPostRequestBody_steps_candidates
    from .workflows_post_request_body_steps_due_date_type import WorkflowsPostRequestBody_steps_dueDateType
    from .workflows_post_request_body_steps_group_review import WorkflowsPostRequestBody_steps_groupReview
    from .workflows_post_request_body_steps_type import WorkflowsPostRequestBody_steps_type

@dataclass
class WorkflowsPostRequestBody_steps(Parsable):
    # (Displayed in the UI when selecting Reviewers/Approvers for a step) Defines the users, roles, or companies to specify for this step of the approval workflow. At least one of the fields (``users``, ``roles``, or ``companies``) must be provided.
    candidates: Optional[WorkflowsPostRequestBody_steps_candidates] = None
    # Specifies how the due date is calculated for this step. It works together with duration.This field applies only to ``REVIEWER`` and ``APPROVER`` steps.Possible values:- ``CALENDAR_DAY`` (default): the due date includes all calendar days, including weekends and holidays.- ``WORKDAY``: the due date excludes weekends and project holidays.
    due_date_type: Optional[WorkflowsPostRequestBody_steps_dueDateType] = None
    # (``Time allowed`` in the UI) The number of days allocated to complete this step.This field applies only to ``REVIEWER`` and ``APPROVER`` steps. It is used to calculate the due date based on the selected ``dueDateType``.Valid range: ``1–99``.
    duration: Optional[int] = None
    # (``Reviewer Type`` in the UI) Defines whether multiple reviewers can participate in this step and how their responses are handled. It applies only to ``REVIEWER`` steps.
    group_review: Optional[WorkflowsPostRequestBody_steps_groupReview] = None
    # The name of the step, as defined in the workflow. It appears in the UI and is used in workflow configuration. Maximum length: 255 characters.Max length: 255
    name: Optional[str] = None
    # Indicates the step type in the workflow. Possible values:- ``INITIATOR``: the first step. It typically represents the person who launches the review.- ``REVIEWER``: an intermediate step. It allows one or more reviewers to evaluate the files.- ``APPROVER``: the final step. It represents the decision maker who approves or rejects the files.
    type: Optional[WorkflowsPostRequestBody_steps_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostRequestBody_steps:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostRequestBody_steps
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostRequestBody_steps()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflows_post_request_body_steps_candidates import WorkflowsPostRequestBody_steps_candidates
        from .workflows_post_request_body_steps_due_date_type import WorkflowsPostRequestBody_steps_dueDateType
        from .workflows_post_request_body_steps_group_review import WorkflowsPostRequestBody_steps_groupReview
        from .workflows_post_request_body_steps_type import WorkflowsPostRequestBody_steps_type

        from .workflows_post_request_body_steps_candidates import WorkflowsPostRequestBody_steps_candidates
        from .workflows_post_request_body_steps_due_date_type import WorkflowsPostRequestBody_steps_dueDateType
        from .workflows_post_request_body_steps_group_review import WorkflowsPostRequestBody_steps_groupReview
        from .workflows_post_request_body_steps_type import WorkflowsPostRequestBody_steps_type

        fields: dict[str, Callable[[Any], None]] = {
            "candidates": lambda n : setattr(self, 'candidates', n.get_object_value(WorkflowsPostRequestBody_steps_candidates)),
            "dueDateType": lambda n : setattr(self, 'due_date_type', n.get_enum_value(WorkflowsPostRequestBody_steps_dueDateType)),
            "duration": lambda n : setattr(self, 'duration', n.get_int_value()),
            "groupReview": lambda n : setattr(self, 'group_review', n.get_object_value(WorkflowsPostRequestBody_steps_groupReview)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(WorkflowsPostRequestBody_steps_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("candidates", self.candidates)
        writer.write_enum_value("dueDateType", self.due_date_type)
        writer.write_int_value("duration", self.duration)
        writer.write_object_value("groupReview", self.group_review)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("type", self.type)
    




from enum import Enum

class WithTransmittalGetResponse_status(str, Enum):
    SENDING = "SENDING",
    Recipients = "recipients",
    ExternalMembers = "externalMembers",
    COMPLETED = "COMPLETED",
    FAILED = "FAILED",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .naming_standards_get_response_definition_fields import NamingStandardsGetResponse_definition_fields
    from .naming_standards_get_response_definition_metadata import NamingStandardsGetResponse_definition_metadata

@dataclass
class NamingStandardsGetResponse_definition(Parsable):
    # A specified character, separating between multiple fields of a file name.Possible values:- ``-``: Hyphen.- ``_``: Underscore.- ``.``: Point.Note that you cannot use the delimiter within a field name.
    delimiter: Optional[str] = None
    # A list of objects defining a file's field names.Note that the order of the objects that the endpoint returns in the response corresponds to the order of the fields that you set for the naming standard in the UI.
    fields: Optional[list[NamingStandardsGetResponse_definition_fields]] = None
    # A list of objects defining a file's related attributes. These attributes do not appear in the file name.
    metadata: Optional[list[NamingStandardsGetResponse_definition_metadata]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> NamingStandardsGetResponse_definition:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: NamingStandardsGetResponse_definition
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return NamingStandardsGetResponse_definition()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .naming_standards_get_response_definition_fields import NamingStandardsGetResponse_definition_fields
        from .naming_standards_get_response_definition_metadata import NamingStandardsGetResponse_definition_metadata

        from .naming_standards_get_response_definition_fields import NamingStandardsGetResponse_definition_fields
        from .naming_standards_get_response_definition_metadata import NamingStandardsGetResponse_definition_metadata

        fields: dict[str, Callable[[Any], None]] = {
            "delimiter": lambda n : setattr(self, 'delimiter', n.get_str_value()),
            "fields": lambda n : setattr(self, 'fields', n.get_collection_of_object_values(NamingStandardsGetResponse_definition_fields)),
            "metadata": lambda n : setattr(self, 'metadata', n.get_collection_of_object_values(NamingStandardsGetResponse_definition_metadata)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("delimiter", self.delimiter)
        writer.write_collection_of_object_values("fields", self.fields)
        writer.write_collection_of_object_values("metadata", self.metadata)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.api_client_builder import enable_backing_store_for_serialization_writer_factory, register_default_deserializer, register_default_serializer
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.serialization import ParseNodeFactoryRegistry, SerializationWriterFactoryRegistry
from kiota_serialization_form.form_parse_node_factory import FormParseNodeFactory
from kiota_serialization_form.form_serialization_writer_factory import FormSerializationWriterFactory
from kiota_serialization_json.json_parse_node_factory import JsonParseNodeFactory
from kiota_serialization_json.json_serialization_writer_factory import JsonSerializationWriterFactory
from kiota_serialization_multipart.multipart_serialization_writer_factory import MultipartSerializationWriterFactory
from kiota_serialization_text.text_parse_node_factory import TextParseNodeFactory
from kiota_serialization_text.text_serialization_writer_factory import TextSerializationWriterFactory
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .bim360.bim360_request_builder import Bim360RequestBuilder
    from .construction.construction_request_builder import ConstructionRequestBuilder
    from .cost.cost_request_builder import CostRequestBuilder
    from .data_connector.data_connector_request_builder import DataConnectorRequestBuilder
    from .hq.hq_request_builder import HqRequestBuilder

class BaseACCclient(BaseRequestBuilder):
    """
    The main entry point of the SDK, exposes the configuration and the fluent API.
    """
    def __init__(self,request_adapter: RequestAdapter) -> None:
        """
        Instantiates a new BaseACCclient and sets the default values.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        if request_adapter is None:
            raise TypeError("request_adapter cannot be null.")
        super().__init__(request_adapter, "{+baseurl}", None)
        register_default_serializer(JsonSerializationWriterFactory)
        register_default_serializer(TextSerializationWriterFactory)
        register_default_serializer(FormSerializationWriterFactory)
        register_default_serializer(MultipartSerializationWriterFactory)
        register_default_deserializer(JsonParseNodeFactory)
        register_default_deserializer(TextParseNodeFactory)
        register_default_deserializer(FormParseNodeFactory)
        if not self.request_adapter.base_url:
            self.request_adapter.base_url = "https://developer.api.autodesk.com"
        self.path_parameters["base_url"] = self.request_adapter.base_url
    
    @property
    def bim360(self) -> Bim360RequestBuilder:
        """
        The bim360 property
        """
        from .bim360.bim360_request_builder import Bim360RequestBuilder

        return Bim360RequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def construction(self) -> ConstructionRequestBuilder:
        """
        The construction property
        """
        from .construction.construction_request_builder import ConstructionRequestBuilder

        return ConstructionRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def cost(self) -> CostRequestBuilder:
        """
        The cost property
        """
        from .cost.cost_request_builder import CostRequestBuilder

        return CostRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def data_connector(self) -> DataConnectorRequestBuilder:
        """
        The dataConnector property
        """
        from .data_connector.data_connector_request_builder import DataConnectorRequestBuilder

        return DataConnectorRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def hq(self) -> HqRequestBuilder:
        """
        The hq property
        """
        from .hq.hq_request_builder import HqRequestBuilder

        return HqRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .actions_get_response_rules import ActionsGetResponse_rules
    from .actions_get_response_transforms import ActionsGetResponse_transforms

@dataclass
class ActionsGetResponse(Parsable):
    # The name of the action that you can perform on the item.
    name: Optional[str] = None
    # The list of conditions that are required in order to be able to perform the action. For example, you can only open a PCO's budgetStatus and costStatus when they are in a draft state.
    rules: Optional[list[ActionsGetResponse_rules]] = None
    # The list of changes that you can perform on the associated item.
    transforms: Optional[list[ActionsGetResponse_transforms]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ActionsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ActionsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ActionsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .actions_get_response_rules import ActionsGetResponse_rules
        from .actions_get_response_transforms import ActionsGetResponse_transforms

        from .actions_get_response_rules import ActionsGetResponse_rules
        from .actions_get_response_transforms import ActionsGetResponse_transforms

        fields: dict[str, Callable[[Any], None]] = {
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "rules": lambda n : setattr(self, 'rules', n.get_collection_of_object_values(ActionsGetResponse_rules)),
            "transforms": lambda n : setattr(self, 'transforms', n.get_collection_of_object_values(ActionsGetResponse_transforms)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("rules", self.rules)
        writer.write_collection_of_object_values("transforms", self.transforms)
    




from enum import Enum

class GetSortQueryParameterType(str, Enum):
    Name = "name",
    NameAsc = "name asc",
    Desc = "desc",
    SortFieldAscOrDesc = "sort=<field> [asc or desc]",
    LastUpdatedAt = "lastUpdatedAt",
    UpdatedByName = "updatedByName",
    SortNameAsc = "sort=name asc",
    SortLastUpdatedAtDesc = "sort=lastUpdatedAt desc",
    LastUpdatedAtAsc = "lastUpdatedAt asc",
    UpdatedByNameAsc = "updatedByName asc",
    NameDesc = "name desc",
    LastUpdatedAtDesc = "lastUpdatedAt desc",
    UpdatedByNameDesc = "updatedByName desc",

